#include "mxhtsp.h"
#include "mxhtspd.h"

int time_btn_long = TIME_BTN_LONG_PRESSED;
int time_chk_part = TIME_CHECK_PARTITION;

disk_info disk[2];
check_list clist[10];

static const char *dir="C:\\\"Program Files\"\\MOXA\\mxhtsp\\script";
static const char *action_part_script="action_over_usage";

static void msleep(unsigned long msec){
	Sleep(msec);
}

static void display_led(int fd, int led_num, int times, int sleep){
	int i;

	for (i = 0; i < times; i++) {
			mxhtsp_set_led(fd, led_num, 1);
			msleep(sleep);
			mxhtsp_set_led(fd, led_num, 0);
			msleep(sleep);
		}
}

static int execute_script(char* action_script, int param){
	HINSTANCE	hInstance;

	hInstance=ShellExecuteA(NULL, "open", "cscript.exe", action_script, NULL, SW_SHOWNORMAL); 
	if((int)hInstance<32){
		printf("ShellExecute fail.Error=%d\r\n",hInstance);
	}

	return 0;
}

static int read_conf(char *path, check_list *cl, int cl_num) 
{
	FILE *fp;
	char buf[LINE_MAX];
    char *delim=" \n";
	char *token = NULL;
	int i = 0;

	if (!(fp = fopen(path, "r"))) {
		perror("read_conf");
	}
	for (i = 0; i < cl_num; i++) {

		if (!fgets(buf, LINE_MAX, fp)) {
			break;
		}
		if (!(token = strtok(buf, delim))) {
			goto err;
		}

		if (strncmp(token, "#", 1) == 0) {
			i--;
			continue;
		} else {
			cl[i].partition_name = _strdup(token);
		}

		if (!(token = strtok(NULL, delim))) {
			goto err;
		}
		cl[i].used_percent = atoi(token);

		if ((cl[i].used_percent > 100) || (cl[i].used_percent < 0)) {
			printf("<ERROR> %s setup a wrong used_percent %d\n", path, cl[i].used_percent); 
			return -1;
		}
		printf("partition_name=%s used_percent=%d\n", cl[i].partition_name, cl[i].used_percent);

	}

	fclose(fp);
	return i;

err:
	fclose(fp);
	printf("%s: parsing %s fail, content=%s\n", __FUNCTION__, path, buf); 
	return -1;
}


static void handle_disk_plugged(int fd, int num){
	int			ret;
	char		action_script[32];
	
	ret = mxhtsp_is_disk_plugged(fd, num); 
	if ((ret == 1) && (disk[num-1].plugged == 0)) {
		printf("mxhtspd: Disk %d status: unplugged -> plugged\n", num);
	}
	else if ((ret == 0) && (disk[num-1].plugged == 1)) {
		printf("mxhtspd: Disk %d status: plugged -> unplugged\n", num);
		/* 
		 * Note: You should umount partiton before disk is unplugged !
		 * For example, add 'umount' at /etc/mxhtsp/btnx_action and press button before unplugging disk
		 * Here is just a prevention for operation mistake 
		 */
		sprintf(action_script,"action_disk%d_unplugged.vbs",num);
		execute_script(action_script,0);
	} 
	disk[num-1].plugged = ret;
}

static void handle_disk_busy(int fd, int num){

	if (mxhtsp_is_disk_busy(fd, num) == 1){ 
		mxhtsp_set_led(fd, num, 1);
	}
	else{ 
		mxhtsp_set_led(fd, num, 0);
	}
}

static int handle_button_pressed(int fd, int num){

	int		now_press;
	int		*ta = &disk[num-1].btn_time_accum;
	int		*btn_switch_pressed = &disk[num-1].btn_swtich_pressed;
	int		*btn_last_pressed = &disk[num-1].btn_last_pressed;
	char	action_script[32];

	now_press = mxhtsp_is_button_pressed(fd, num); 

	/*Check if a new press event is occured*/
	if (now_press == 1 && (*btn_last_pressed == 0)) { 
		*btn_switch_pressed = 1;	/*Record the button from unpressed to pressed*/
		*btn_last_pressed = 1;		/*Record the last press action*/
	} /*If button is continuously pressed, then accumulate the press time*/
	else if (now_press == 1 && (*btn_last_pressed == 1)){
		if (*btn_switch_pressed) {
			*ta += SLEEP_TIME;
			if (*ta > time_btn_long) { 
				sprintf(action_script,"action_btn%d_pressed_long.vbs",num);
				execute_script(action_script, 0);
				printf("long press\r\n");
				display_led(fd,num,3,500);
				*btn_switch_pressed = 0;
				*btn_last_pressed = 1;
				*ta = 0;
			} 
		}
	}/*If the press action is not continuous, then a shot press action is occured*/
	else if (now_press == 0) {
		if (*btn_switch_pressed) {
			//execute_script("action_btn_pressed_short", num);
			sprintf(action_script,"action_btn%d_pressed_short.vbs",num);
			execute_script(action_script, 0);
			printf("short press\r\n");
			display_led(fd,num,1,500);
			*btn_switch_pressed = 0;
		}
		*btn_last_pressed = 0;
	} 
	else {
		printf("mxhtsp_is_button_pressed error now_press=%d\n", now_press);
		*btn_switch_pressed = 0;
		*btn_last_pressed = 0;
		*ta = 0;
	}
	return 0;
}

static void check_part_usage(check_list* cl, int cl_num) 
{
	int i;
	int percent;
	char *script;
	check_list *c;
	
	for (i = 0; i < cl_num; i++) {

		c = &cl[i];
		percent = mxhtsp_check_partition_usage(c->partition_name); 

		if (percent == -1) {
			printf("check partition usage => %s doesn't exist\n", c->partition_name); 
		}
		else if (percent > c->used_percent) 
		{
			script = malloc(strlen(dir) + strlen(action_part_script) + strlen(c->partition_name) + 2);
			printf("%s use %d%% > %d%% => %s %s\n", 
					c->partition_name, percent, c->used_percent, 
					action_part_script, c->partition_name);

			/*Prepare the script name*/
			sprintf(script, "%s\\%s_%c.vbs", dir, action_part_script, c->partition_name[0]);
			printf("script=%s\r\n",script);

			/*Execute the script*/
			execute_script(script,0);//system(script);
			free(script);
		} 
	}
}

int main(int argc, char *argv[]) {
	int fd;
	int clist_num;
	int cnt = 0;
	int i;

	/* read config */
	clist_num = read_conf("mxhtspd.conf", clist, sizeof(clist));
	//clist_num=2;
	printf("clist_num=%d\r\n",clist_num);
	for(i=0;i<clist_num;i++){
		printf("clist[%d].partition_name=%s,clist[%d].used_percent=%d\r\n",i,clist[i].partition_name,i,clist[i].used_percent);
	}

	fd = mxhtsp_open();
	if (fd < 0){
		printf("mxhtspd: can't open device\n");
	}

	cnt = time_chk_part/SLEEP_TIME; 
	printf("cnt=%d\r\n",cnt);

	while (1) {

		/*Check disk is plugged or not*/
		handle_disk_plugged(fd, 1);
		handle_disk_plugged(fd, 2);

		/*Check disk is busy or not*/
		handle_disk_busy(fd, 1);
		handle_disk_busy(fd, 2);

		/*Check button is pressed or not*/
		handle_button_pressed(fd, 1);
		handle_button_pressed(fd, 2);

		/*Check partition usage*/
		if (--cnt == 0) {
			check_part_usage(clist, clist_num);
			cnt = time_chk_part/SLEEP_TIME; 
		}

		msleep(SLEEP_TIME);
	}
	mxhtsp_close(fd);

	return 0;
}

